@@ -16,9 +16,14 @@
 #' |silt     |Proportion of silt particles (= 0.002 mm and = 0.05 mm) in the fine earth fraction |g/kg           |                10|g/100g (%)         |
 #' |soc      |Soil organic carbon content in the fine earth fraction                             |dg/kg          |                10|g/kg               |
 #' |ocd      |Organic carbon density                                                             |hg/m^3         |                10|kg/m^3             |
-#' |ocs      |Organic carbon stocks                                                              |t/ha           |                10|kg/m^2             |
+#' |ocs      |Organic carbon stocks (0-30cm depth interval only)                                 |t/ha           |                10|kg/m^2             |
 #' 
-#' SoilGrids predictions are made for the six standard depth intervals specified in the GlobalSoilMap IUSS working group and its specifications. The depth intervals returned are: \code{"0-5cm", "5-15cm", "15-30cm", "30-60cm", "60-100cm", "100-200cm"} and the properties returned are \code{"bdod", "cec", "cfvo", "clay", "nitrogen", "phh2o", "sand", "silt", "soc"} -- each with 5th, 50th, 95th, mean and uncertainty values. The uncertainty values are the ratio between the inter-quantile range (90% prediction interval width) and the median : `(Q0.95-Q0.05)/Q0.50.` Point data requests are made through \code{properties/query} endpoint of the [SoilGrids v2.0 REST API](https://www.isric.org/explore/soilgrids/faq-soilgrids). Please check ISRIC's data policy, disclaimer and citation: \url{https://www.isric.org/about/data-policy}.
+#' SoilGrids predictions are made for the six standard depth intervals specified in the GlobalSoilMap IUSS working group and its specifications. The default depth 
+#' intervals returned are (in centimeters): `"0-5"`, `"5-15"`, `"15-30"`, `"30-60"`, `"60-100"`, `"100-200"` for the properties `"bdod"`, `"cec"`, `"cfvo"`, 
+#' `"clay"`, `"nitrogen"`, `"phh2o"`, `"sand"`, `"silt"`, `"soc"`, `"ocd"`--each with 5th, 50th, 95th, mean and uncertainty values. Soil organic carbon stocks (0-30cm) (`variables="ocs"`) are returned only for `depth_intervals="0-30"`. The uncertainty values are the ratio
+#' between the inter-quantile range (90% prediction interval width) and the median : `(Q0.95-Q0.05)/Q0.50.` All values are converted from "mapped" to "conventional" 
+#' based on above table conversion factors. Point data requests are made through `"properties/query"` endpoint of the [SoilGrids v2.0 REST API](https://www.isric.org/explore/soilgrids/faq-soilgrids). 
+#' Please check ISRIC's data policy, disclaimer and citation: \url{https://www.isric.org/about/data-policy}.
 #' 
 #' Find out more information about the SoilGrids and GlobalSoilMap products here: 
 #' 
@@ -27,8 +32,10 @@
 #' 
 #' @references Poggio, L., de Sousa, L. M., Batjes, N. H., Heuvelink, G. B. M., Kempen, B., Ribeiro, E., and Rossiter, D.: SoilGrids 2.0: producing soil information for the globe with quantified spatial uncertainty, SOIL, 7, 217-240, 2021. \doi{https://doi.org/10.5194/soil-7-217-2021}
 #' @importFrom utils packageVersion
-#' @param x A \code{data.frame} containing 3 columns referring to site ID, latitude and longitude.
-#' @param loc.names Optional: Column names referring to site ID, latitude and longitude. Default: \code{c("id","lat","lon")}
+#' @param x A `data.frame` containing 3 columns referring to site ID, latitude and longitude.
+#' @param loc.names Optional: Column names referring to site ID, latitude and longitude. Default: `c("id", "lat", "lon")`
+#' @param depth_intervals Default: `"0-5"`, `"5-15"`, `"15-30"`, `"30-60"`, `"60-100"`, `"100-200"`
+#' @param variables Default: `"bdod"`, `"cec"`, `"cfvo"`, `"clay"`, `"nitrogen"`, `"phh2o"`, `"sand"`, `"silt"`, `"soc"`, `"ocd"`
 #' @param verbose Print messages? Default: `FALSE`
 #' @param progress logical, give progress when iterating over multiple requests; Default: `FALSE`
 #' 
@@ -39,19 +46,47 @@
 #' @examplesIf curl::has_internet()
 #' @examples
 #' \dontrun{
-#' 
+#'   library(aqp)
+#'   
 #'   your.points <- data.frame(id  = c("A", "B"), 
 #'                            lat = c(37.9, 38.1), 
 #'                            lon = c(-120.3, -121.5), 
 #'                            stringsAsFactors = FALSE)
 #'   x <- try(fetchSoilGrids(your.points))
 #'  
-#'   library(aqp)
 #'   if (!inherits(x, 'try-error'))
-#'    plotSPC(x, name = NA, color = "socQ50")
+#'    aqp::plotSPC(x, name = NA, color = "socQ50")
+#'  
+#'   # organic carbon stocks use 0-30cm interval
+#'   y <- try(fetchSoilGrids(your.points[1, ], 
+#'                           depth_interval = c("0-5", "0-30", "5-15", "15-30"),
+#'                           variables = c("soc", "bdod", "ocd", "ocs")))
+#'                           
+#'   # extract horizons from a SoilProfileCollection where horizon 2 overlaps 1, 3, and 4
+#'   h <- aqp::horizons(y)
+#'   
+#'   # "ocs" (organic carbon stock 0-30cm interval)
+#'   h[2, ]
+#'   
+#'   h$thickness_meters <- ((h$hzdepb - h$hzdept) / 100)
+#'
+#'   # estimate "ocs" from modeled organic carbon and bulk density in 0-5, 5-15, 15-30 intervals
+#'   #  (sum the product of soc, bdod, and thickness in meters)
+#'   #  (1 gram per cubic decimeter = 1 kilogram per cubic meter)
+#'   sum(h$socmean * h$bdodmean * h$thickness_meters, na.rm = TRUE)
+#'   
+#'   # estimate "ocs" from modeled organic carbon density in 0-5, 5-15, 15-30 intervals
+#'   #  (sum the product of "ocd" and thickness in meters)
+#'   sum(h$ocdmean * h$thickness_meters, na.rm = TRUE)
+#'  
 #' }
 fetchSoilGrids <- function(x,
                            loc.names = c("id", "lat", "lon"),
+                           depth_intervals = c("0-5", "5-15", "15-30", 
+                                               "30-60", "60-100", "100-200"),
+                           variables = c("bdod", "cec", "cfvo", "clay", 
+                                         "nitrogen", "phh2o", "sand", "silt", 
+                                         "soc", "ocd"),
                            verbose = FALSE,
                            progress = FALSE) {
   
@@ -137,17 +172,33 @@ fetchSoilGrids <- function(x,
     }
     
     # create new horizon data, merge in each property using standard depth labels
-    depth.intervals <-  c("0-5", "5-15", "15-30", "30-60", "60-100", "100-200")
-    hz.data <- data.frame(id = id, latitude = lat, longitude = lon, label = depth.intervals, stringsAsFactors = FALSE)
+    depth.intervals <- match.arg(gsub(" ", "", depth_intervals), 
+                                 c("0-5", "5-15", "15-30", "0-30",
+                                   "30-60", "60-100", "100-200"), 
+                                 several.ok = TRUE)
+    hz.data0 <- data.frame(id = id, latitude = lat, longitude = lon, 
+                          label = depth.intervals, stringsAsFactors = FALSE)
 
     # values returned for each layer include the following properties
-    data.types <- c("bdod", "cec", "cfvo", "clay", "nitrogen", "phh2o", "sand", "silt", "soc")
+    data.types <- match.arg(gsub(" ", "", variables), c("bdod", "cec", "cfvo", 
+                                                        "clay", "nitrogen", "phh2o",
+                                                        "sand", "silt", "soc",
+                                                        "ocd", "ocs"), several.ok = TRUE)
     
     # numeric values are returned as integers that need to be scaled to match typical measurement units
-    data.factor <- c(0.01, 0.1, 0.1, 0.1, 0.01, 0.1, 0.1, 0.1, 0.1)
-    
+    data.factors <- c(bdod = 0.01, cec = 0.1, cfvo = 0.1, clay = 0.1, nitrogen = 0.01, 
+                      phh2o = 0.1, sand = 0.1, silt = 0.1, soc = 0.1, ocd = 0.1, ocs = 0.1)
+    data.factor <- data.factors[data.types]
+    hz.data <- hz.data0
+    all_x <- FALSE
     for (d in 1:length(data.types)) {
-      hz.data <- merge(hz.data, .extractSGLayerProperties(jres, data.types[d], data.factor[d]), by = "label")
+      hz.data <- merge(
+          merge(hz.data0, hz.data, by = c("label", "id", "latitude", "longitude"),
+                sort = FALSE, all.x = TRUE),
+          .extractSGLayerProperties(jres, data.types[d], data.factor[d]),
+          all.x = all_x, sort = FALSE, by = "label"
+        )
+      all_x <- TRUE
     }
     
     rownames(hz.data) <- NULL
@@ -160,16 +211,18 @@ fetchSoilGrids <- function(x,
   }
   
   # combine horizon data together
-  spc <- data.table::rbindlist(res, fill = TRUE)
+  spc <- data.frame(data.table::rbindlist(res, fill = TRUE))
   
   # calculate top and bottom depths from label
   labelsplit <- strsplit(as.character(spc$label), split = "-")
   
+  #  note sort order of labels may differ from sort order of depths
   spc$hzdept <- as.numeric(lapply(labelsplit, function(x) x[1]))
   spc$hzdepb <- as.numeric(lapply(labelsplit, function(x) x[2]))
+  spc <- spc[order(spc$id, spc$hzdept, spc$hzdepb),]
   
   # promote to SoilProfileCollection
-  depths(spc) <- id ~ hzdept + hzdepb
+  aqp::depths(spc) <- id ~ hzdept + hzdepb
   
   # move location information to site
   aqp::site(spc) <- ~ longitude + latitude
@@ -185,7 +238,7 @@ fetchSoilGrids <- function(x,
   
   # merge the rest of the sf object into the site table 
   if (spatial_input) {
-    site(spc) <- cbind(id = 1:nrow(x), sf::st_drop_geometry(x))
+    aqp::site(spc) <- cbind(id = 1:nrow(x), sf::st_drop_geometry(x))
   }
   
   return(spc)
@@ -202,23 +255,12 @@ fetchSoilGrids <- function(x,
   
   out[["values"]] <- cbind(sgvalues, uncertainty)
   
-  if (nrow(out[["values"]]) == 0) {
-    out <- data.frame(range = NA, 
-                      label = c("0-5", "5-15", "15-30", 
-                                "30-60", "60-100", "100-200"), 
-                      values = data.frame(
-                          Q0.05 = rep(NA_real_, 6), Q0.5 = rep(NA_real_, 6), Q0.95 = rep(NA_real_, 6),
-                          mean = rep(NA_real_, 6), uncertainty = rep(NA_real_, 6)
-                       ))
-  }
-  # } else {
-    # fix names and labels for downstream
-    out <- out[,colnames(out)[grep("range", colnames(out), invert = TRUE)]]
-    out <- cbind(label = gsub("cm", "", out$label), values = out$values)
-    colnames(out) <- gsub("\\.Q0\\.", "Q", colnames(out))
-    colnames(out) <- gsub("Q5", "Q50", colnames(out))
-    colnames(out) <- gsub("values", x, colnames(out))
-    colnames(out) <- gsub("\\.", "", colnames(out))
-  # }
+  # fix names and labels for downstream
+  out <- out[, colnames(out)[grep("range", colnames(out), invert = TRUE)]]
+  out <- cbind(label = gsub("cm", "", out$label), values = out$values)
+  colnames(out) <- gsub("\\.Q0\\.", "Q", colnames(out))
+  colnames(out) <- gsub("Q5", "Q50", colnames(out))
+  colnames(out) <- gsub("values", x, colnames(out))
+  colnames(out) <- gsub("\\.", "", colnames(out))
   return(out)
 }
